module Calculate
  def self.is_prime?(n)
    n = n.to_i
    return false if n < 2

    # http://www.noulakaz.net/weblog/2007/03/18/a-regular-expression-to-check-for-prime-numbers/
    # ("1" * n) !~ /^1?$|^(11+?)\1+$/

    # [*2...n].each { |block_n| return false unless n % block_n }

    # Suppose one number is a factor of N and that it is smaller than the square-root of the number N.
    # Then the second factor must be larger than the square-root.
    [*2..Math.sqrt(n)].reject { |block_n| n % block_n > 0 }.empty?
  end

  def self.next_prime(from)
    # Don't start from 1, because silly.
    from += 1 until self.is_prime?(from)
    from
  end

  def self.n_primes(n = 10)
    last_prime = 0
    primes = []

    while n > 0 do
      last_prime = self.next_prime(last_prime + 1)
      primes << last_prime
      n -= 1
    end

    primes
  end
end

module Display
  def self.for_primes(count, size)
    "%#{size}d |" * count
  end

  def self.table(primes)
    # Get largest number - to ensure the table cells are even and fit.
    last_element = primes.sort.reverse.first
    element_size = (last_element*last_element).to_s.length
    element_size = element_size < 4 ? 4 : element_size + 1

    $stdout.printf "%#{element_size+2}s #{ self.for_primes(primes.count, element_size) }\n", '', *primes
    $stdout.printf "%#{element_size+2}s #{ "%#{element_size}s |" * primes.count }\n", '', *primes.map { |s| '-' * element_size }

    primes.each do |prime|
      $stdout.printf "%#{element_size+1}d -#{self.for_primes(primes.count, element_size) }\n", prime, *primes.map { |n| n*prime }
    end
  end
end

module PrimeTable
  class Optimus
    attr_accessor :input, :output, :primes

    include Calculate
    include Display

    # Get and parse user input.

    def initialize
      @primes = []
    end

    def get_input
      $stdout.puts "'q' to escape"
      $stdout.print "Enter the required number of primes, or press enter to display the first #{@primes.count > 0 ? @primes.count : 10}: "
      parse_input $stdin.gets.chomp
    end

    def parse_input(input)
      return if input == "q"

      if input.empty?
        input = @primes.empty? ? 10 : @primes.count
      end

      input = input.to_i
      if input < 2
        $stdout.puts "Orly?  Here you go! []"
        return get_input
      end

      @primes = Calculate::n_primes(input)
      Display::table(@primes)

      return get_input
    end

  end
end

